#include "../twn_timer.h"
#include "twn_engine_context_c.h"

#include <SDL_messagebox.h>

#include <signal.h>
#include <stdint.h>
#include <time.h>
#include <unistd.h>
#include <stdbool.h>


static timer_t            timerid;
static struct sigaction   sa;
static struct sigevent    sev;

static bool created;


/* stop application */
static void sanity_timer_handler(int sig, siginfo_t *si, void *uc) {
    (void)uc; (void)sig; (void)si;
    SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR,
                             "sanity timer", "Your game tick exceeded its allocated time, application is closing",
                             ctx.window);
    raise(SIGKILL);
}


bool start_sanity_timer(uint64_t milliseconds_to_expire) {
    if (!created) {
        sa.sa_flags = SA_SIGINFO;
        sa.sa_sigaction = sanity_timer_handler;
        sigemptyset(&sa.sa_mask);
        if (sigaction(SIGRTMIN, &sa, NULL) == -1)
            goto ERR_SIGACTION;

       sev.sigev_notify = SIGEV_SIGNAL;
       sev.sigev_signo = SIGRTMIN;
       sev.sigev_value.sival_ptr = &timerid;
       if (timer_create(CLOCK_MONOTONIC, &sev, &timerid) == -1)
            goto ERR_TIMERCREATE;

        created = true;

    ERR_TIMERCREATE:
    // ERR_SIGPROCMASK:
    ERR_SIGACTION:
        return false;
    }

    struct itimerspec its = {0};
    its.it_value.tv_sec = milliseconds_to_expire / 1000;
    its.it_value.tv_nsec = (milliseconds_to_expire * 1000000 % 1000000000);
    if (timer_settime(timerid, 0, &its, NULL) == -1)
        return false;

    return true;
}


bool end_sanity_timer(void) {
    struct itimerspec its = {0};
    its.it_value.tv_sec = 0;
    its.it_value.tv_nsec = 0;
    if (timer_settime(timerid, 0, &its, NULL) == -1)
        return false;

    return false;
}
